dnl -*- shell-script -*-
dnl
dnl Copyright (c) 2001-2002 The Trustees of Indiana University.  
dnl                         All rights reserved.
dnl Copyright (c) 1998-2001 University of Notre Dame. 
dnl                         All rights reserved.
dnl Copyright (c) 1994-1998 The Ohio State University.  
dnl                         All rights reserved.
dnl 
dnl This file is part of the LAM/MPI software package.  For license
dnl information, see the LICENSE file in the top level directory of the
dnl LAM/MPI source distribution.
dnl
dnl $Id: lam_config_threads.m4,v 1.12 2003/06/02 02:30:52 jsquyres Exp $
dnl

define(LAM_CONFIG_THREADS,[
#
# Arguments: none
#
# Dependencies: None
#
# Modifies:
#  none - see called tests
#
# configure threads
#


# create templates
AH_TEMPLATE([LAM_HAVE_SOLARIS_THREADS], 
    [Do we want to use native Solaris threads])
AH_TEMPLATE([LAM_HAVE_POSIX_THREADS], 
    [Do we want to use POSIX threads])

#
# Ask if we want to disable -pthread test
#
AC_MSG_CHECKING([whether to disable -pthread check])
AC_ARG_WITH(dash-pthread,
	    AC_HELP_STRING([--with(out)-dash-pthread],
	       [disable use of -pthread compiler option]))
if test "$with_dash_pthread" = "yes"; then
    dash_pthread_result="no"
    lam_no_dash_pthread=0
elif test "$with_dash_pthread" = "no"; then
    dash_pthread_result="yes"
    lam_no_dash_pthread=1
else
    dash_pthread_result="no"
    lam_no_dash_pthread=0
fi
AC_MSG_RESULT($dash_pthread_result)
unset dash_pthread_result

#
# Check for thread types - add your type here...
#
libs_SAVE="$LIBS"
AC_CHECK_LIB(thread, thr_create, HAVE_SOLARIS_THREADS=1, 
    HAVE_SOLARIS_THREADS=0)
LIBS="$libs_SAVE"
libs_SAVE="$LIBS"
AC_CHECK_LIB(pthread, pthread_create, HAVE_POSIX_THREADS=1,
    HAVE_POSIX_THREADS=0)
LIBS="$libs_SAVE"

#
# Note that some systems don't have pthread support in -lpthread, but
# in -pthread instead.  So check there as well.  We'll check more
# thorougly below to see if we should end up using -lpthread or
# -pthread.
#

if test "$lam_no_dash_pthread" = "0"; then
if test "$HAVE_POSIX_THREADS" = "0"; then
    LIBS="$LIBS -pthread"
    AC_CHECK_FUNC(pthread_create, HAVE_POSIX_THREADS=1,
	HAVE_POSIX_THREADS="0")
    LIBS="$libs_SAVE"
fi
fi


#
# Ask what threading we want (allow solaris / pthread right now)
#
AC_MSG_CHECKING([for type of thread support])
AC_ARG_WITH(threads, 
  	[  --with-threads          Set thread type (solaris / pthread)],
	[THREAD_TYPE=$withval])

if test "$THREAD_TYPE" = "solaris"; then

    if test "$HAVE_SOLARIS_THREADS" = "0"; then
	AC_MSG_WARN(["*** You have chosen Solaris threads, which are not"])
	AC_MSG_WARN(["*** available on your system "])
	AC_MSG_ERROR(["*** Can not continue"])
    fi
elif test "$THREAD_TYPE" = "posix"; then

    if test "$HAVE_POSIX_THREADS" = "0"; then
	AC_MSG_WARN(["*** You have chosen POSIX threads, which are not"])
	AC_MSG_WARN(["*** available on your system "])
	AC_MSG_ERROR(["*** Can not continue"])
    fi
elif test "$THREAD_TYPE" = "no"; then
    THREAD_TYPE="none"
elif test "$THREAD_TYPE" = ""; then

    # Actual logic here - properly set THREAD_TYPE - we go for system
    # optimized where ever possible
    case "$host" in
	*solaris*)
	    if test "$HAVE_SOLARIS_THREADS" = "1"; then
		THREAD_TYPE="solaris"
	    elif test "$HAVE_POSIX_THREADS" = "1"; then
		THREAD_TYPE="posix"
	    else
		THEAD_TYPE="none found"
	    fi
	    ;;
	*)
	    if test "$HAVE_POSIX_THREADS" = "1"; then
		THREAD_TYPE="posix"
	    else
		THREAD_TYPE="none found"
	    fi
	    ;;
    esac
else

    AC_MSG_WARN(["*** You have specified a thread type that I do not"])
    AC_MSG_WARN(["*** understand.  Valid options are posix and solaris"])
    AC_MSG_ERROR(["*** Can not continue."])
fi
AC_MSG_RESULT($THREAD_TYPE)


#
# Ok, now run the configuration for that thread package.
#
# Blah - this should be made better, but I don't know how...
#
if test "$THREAD_TYPE" = "solaris"; then
    AC_DEFINE(LAM_HAVE_SOLARIS_THREADS, 1)
    AC_DEFINE(LAM_HAVE_POSIX_THREADS, 0)
    AC_DEFINE(LAM_THREADS_HAVE_DIFFERENT_PIDS, 0)

    LAM_CONFIG_SOLARIS_THREADS
elif test "$THREAD_TYPE" = "posix"; then
    AC_DEFINE(LAM_HAVE_SOLARIS_THREADS, 0)
    AC_DEFINE(LAM_HAVE_POSIX_THREADS, 1)

    LAM_CONFIG_POSIX_THREADS
elif test "$THREAD_TYPE" = "none"; then
    AC_DEFINE(LAM_HAVE_SOLARIS_THREADS, 0)
    AC_DEFINE(LAM_HAVE_POSIX_THREADS, 0)
else
    cat <<EOF

*****************************************************************************

LAM/MPI was unable to find threading support on your system.  In the near
future, the LAM development team is considering requiring threading support
for proper LAM execution.  This is in part because we are not aware of any
users that do not have thread support - so we need you to e-mail us at
lam@lam-mpi.org and let us know about this problem.

To build this version of LAM/MPI without thread support, re-run
configure with the '--without-threads' option.

*****************************************************************************

EOF
    AC_MSG_ERROR(["*** Can not continue."])
fi
])


########################################################################
#
# Every configuration test must AC_SUBST the following:
#
#  THREADCPPFLAGS
#  THREADLDFLAGS
#  THREADLIBS
#  THREADCXXCPPFLAGS
#  THREADCXXLDFLAGS
#  THREADCXXLIBS
#
# Since only one will be called, this isn't a problem
#
########################################################################
define(LAM_CONFIG_SOLARIS_THREADS,[

    if test "$BASECC" = "cc"; then
	THREADCPPFLAGS="-mt"
	THREADFFLAGS="-mt"
	THREADLDFLAGS="-mt"
	THREADLIBS=""
    else
	THREADCPPFLAGS="-D_REENTRANT"
	THREADFFLAGS=""
	THREADLDFLAGS=""
	THREADLIBS="-lthread"
    fi

    if test "$BASECXX" = "CC"; then
	THREADCXXCPPFLAGS="-mt"
	THREADCXXLDFLAGS="-mt"
	THREADLIBS=""
    elif test "$BASECXX" = "KCC"; then
	THREADCXXCPPFLAGS="--backend -mt"
	THREADCXXLDFLAGS="--backend -mt"
	THREADLIBS=""
    else
	THREADCXXCPPFLAGS="-D_REENTRANT"
	THREADCXXLDFLAGS=""
	THREADLIBS="-lthread"
    fi
])


define(LAM_CONFIG_POSIX_THREADS,[

    # If we have gcc, the -pthread flag may work (many Linux, BSD, and
    # other gcc-based distros add this additional flag in the gcc
    # configuration) and do what we need it to do for both CPPFLAGS
    # and LIBS.  So test to see if the -pthread flag works.

    if test "$lam_no_dash_pthread" = "0" && test "$GCC" = "yes"; then
	# This causes additional output -- from the output, it looks
	# like we're checking for pthread_create a second time (since
	# we already checked for its presence, above).  But it's not
	# worth spending time on just to make the output look right --
	# better to use the standard AC_* macros than to write a
	# custom one just to not show output.  :-)

	AC_MSG_CHECKING([if -pthread works (GNU compilers only)])
	rm -f conftest.c
	cat > conftest.c <<EOF
#line $LINENO "configure"
#include <pthread.h>
char (*f)() = pthread_create;

int main()
{
    return f != pthread_create;
}
EOF
	lam_cmd="$CC $CFLAGS conftest.c -o conftest.exe -pthread $LDFLAGS $LIBS >&5 2> conftest.out"
	echo "configure:$LINENO: $lam_cmd" >&5
	eval $lam_cmd
	lam_status="$?"
	echo "configure:$LINENO: \$? = $lam_status" >&5
	if test "$lam_status" != 0 -o "`grep 'unrecognized option' conftest.out`" != ""; then
	    echo "configure: failed program was: " >&5
	    cat conftest.c >&5
	    THREADCPPFLAGS="-D_REENTRANT"
	    THREADFFLAGS=""
	    THREADLDFLAGS=""
	    THREADLIBS="-lpthread"
	    AC_MSG_RESULT([no])
	else
	    THREADCPPFLAGS="-pthread"
	    THREADFFLAGS="-pthread"
	    THREADLDFLAGS=""
	    THREADLIBS="-pthread"
	    AC_MSG_RESULT([yes])
	fi
	rm -f conftest.c fontest.exe conftest.out
    else
	THREADCPPFLAGS="-D_REENTRANT"
	THREADFFLAGS=""
	THREADLDFLAGS=""
	THREADLIBS="-lpthread"
    fi
    THREADCXXCPPFLAGS="$THREADCPPFLAGS"
    THREADCXXLDFLAGS="$THREADLDFLAGS"
    THREADCXXLIBS="$THREADLIBS"

    #
    # Check to see if threads are brain-dead with their PIDs
    #
    LAM_CHECK_PTHREAD_PIDS

    #
    # Clean up
    #
    unset lam_cmd lam_status
])
